# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.types import (
    LegalEntity,
)
from modern_treasury._utils import parse_date, parse_datetime
from modern_treasury.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestLegalEntities:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: ModernTreasury) -> None:
        legal_entity = client.legal_entities.create(
            legal_entity_type="business",
        )
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: ModernTreasury) -> None:
        legal_entity = client.legal_entities.create(
            legal_entity_type="business",
            addresses=[
                {
                    "country": "country",
                    "line1": "line1",
                    "locality": "locality",
                    "postal_code": "postal_code",
                    "region": "region",
                    "address_types": ["business"],
                    "line2": "line2",
                }
            ],
            bank_settings={
                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "backup_withholding_percentage": 0,
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "enable_backup_withholding": True,
                "live_mode": True,
                "object": "object",
                "privacy_opt_out": True,
                "regulation_o": True,
                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            business_name="business_name",
            citizenship_country="citizenship_country",
            date_formed=parse_date("2019-12-27"),
            date_of_birth=parse_date("2019-12-27"),
            doing_business_as_names=["string"],
            email="email",
            first_name="first_name",
            identifications=[
                {
                    "id_number": "id_number",
                    "id_type": "ar_cuil",
                    "issuing_country": "issuing_country",
                }
            ],
            last_name="last_name",
            legal_entity_associations=[
                {
                    "relationship_types": ["beneficial_owner"],
                    "child_legal_entity": {
                        "addresses": [
                            {
                                "country": "country",
                                "line1": "line1",
                                "locality": "locality",
                                "postal_code": "postal_code",
                                "region": "region",
                                "address_types": ["business"],
                                "line2": "line2",
                            }
                        ],
                        "bank_settings": {
                            "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "backup_withholding_percentage": 0,
                            "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "enable_backup_withholding": True,
                            "live_mode": True,
                            "object": "object",
                            "privacy_opt_out": True,
                            "regulation_o": True,
                            "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                        },
                        "business_name": "business_name",
                        "citizenship_country": "citizenship_country",
                        "date_formed": parse_date("2019-12-27"),
                        "date_of_birth": parse_date("2019-12-27"),
                        "doing_business_as_names": ["string"],
                        "email": "email",
                        "first_name": "first_name",
                        "identifications": [
                            {
                                "id_number": "id_number",
                                "id_type": "ar_cuil",
                                "issuing_country": "issuing_country",
                            }
                        ],
                        "last_name": "last_name",
                        "legal_entity_type": "business",
                        "legal_structure": "corporation",
                        "metadata": {
                            "key": "value",
                            "foo": "bar",
                            "modern": "treasury",
                        },
                        "middle_name": "middle_name",
                        "phone_numbers": [{"phone_number": "phone_number"}],
                        "politically_exposed_person": True,
                        "preferred_name": "preferred_name",
                        "prefix": "prefix",
                        "risk_rating": "low",
                        "suffix": "suffix",
                        "wealth_and_employment_details": {
                            "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "annual_income": 0,
                            "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "employer_country": "employer_country",
                            "employer_name": "employer_name",
                            "employer_state": "employer_state",
                            "employment_status": "employed",
                            "income_country": "income_country",
                            "income_source": "family_support",
                            "income_state": "income_state",
                            "industry": "accounting",
                            "live_mode": True,
                            "object": "object",
                            "occupation": "consulting",
                            "source_of_funds": "alimony",
                            "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "wealth_source": "business_sale",
                        },
                        "website": "website",
                    },
                    "child_legal_entity_id": "child_legal_entity_id",
                    "ownership_percentage": 0,
                    "title": "title",
                }
            ],
            legal_structure="corporation",
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
            middle_name="middle_name",
            phone_numbers=[{"phone_number": "phone_number"}],
            politically_exposed_person=True,
            preferred_name="preferred_name",
            prefix="prefix",
            risk_rating="low",
            suffix="suffix",
            wealth_and_employment_details={
                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "annual_income": 0,
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "employer_country": "employer_country",
                "employer_name": "employer_name",
                "employer_state": "employer_state",
                "employment_status": "employed",
                "income_country": "income_country",
                "income_source": "family_support",
                "income_state": "income_state",
                "industry": "accounting",
                "live_mode": True,
                "object": "object",
                "occupation": "consulting",
                "source_of_funds": "alimony",
                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "wealth_source": "business_sale",
            },
            website="website",
        )
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: ModernTreasury) -> None:
        response = client.legal_entities.with_raw_response.create(
            legal_entity_type="business",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        legal_entity = response.parse()
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: ModernTreasury) -> None:
        with client.legal_entities.with_streaming_response.create(
            legal_entity_type="business",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            legal_entity = response.parse()
            assert_matches_type(LegalEntity, legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        legal_entity = client.legal_entities.retrieve(
            "id",
        )
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.legal_entities.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        legal_entity = response.parse()
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.legal_entities.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            legal_entity = response.parse()
            assert_matches_type(LegalEntity, legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.legal_entities.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: ModernTreasury) -> None:
        legal_entity = client.legal_entities.update(
            id="id",
        )
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: ModernTreasury) -> None:
        legal_entity = client.legal_entities.update(
            id="id",
            addresses=[
                {
                    "country": "country",
                    "line1": "line1",
                    "locality": "locality",
                    "postal_code": "postal_code",
                    "region": "region",
                    "address_types": ["business"],
                    "line2": "line2",
                }
            ],
            bank_settings={
                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "backup_withholding_percentage": 0,
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "enable_backup_withholding": True,
                "live_mode": True,
                "object": "object",
                "privacy_opt_out": True,
                "regulation_o": True,
                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            business_name="business_name",
            citizenship_country="citizenship_country",
            date_formed=parse_date("2019-12-27"),
            date_of_birth=parse_date("2019-12-27"),
            doing_business_as_names=["string"],
            email="email",
            first_name="first_name",
            identifications=[
                {
                    "id_number": "id_number",
                    "id_type": "ar_cuil",
                    "issuing_country": "issuing_country",
                }
            ],
            last_name="last_name",
            legal_structure="corporation",
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
            middle_name="middle_name",
            phone_numbers=[{"phone_number": "phone_number"}],
            politically_exposed_person=True,
            preferred_name="preferred_name",
            prefix="prefix",
            risk_rating="low",
            suffix="suffix",
            wealth_and_employment_details={
                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "annual_income": 0,
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "employer_country": "employer_country",
                "employer_name": "employer_name",
                "employer_state": "employer_state",
                "employment_status": "employed",
                "income_country": "income_country",
                "income_source": "family_support",
                "income_state": "income_state",
                "industry": "accounting",
                "live_mode": True,
                "object": "object",
                "occupation": "consulting",
                "source_of_funds": "alimony",
                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "wealth_source": "business_sale",
            },
            website="website",
        )
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: ModernTreasury) -> None:
        response = client.legal_entities.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        legal_entity = response.parse()
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: ModernTreasury) -> None:
        with client.legal_entities.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            legal_entity = response.parse()
            assert_matches_type(LegalEntity, legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.legal_entities.with_raw_response.update(
                id="",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        legal_entity = client.legal_entities.list()
        assert_matches_type(SyncPage[LegalEntity], legal_entity, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        legal_entity = client.legal_entities.list(
            after_cursor="after_cursor",
            legal_entity_type="business",
            metadata={"foo": "string"},
            per_page=0,
            show_deleted="show_deleted",
        )
        assert_matches_type(SyncPage[LegalEntity], legal_entity, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.legal_entities.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        legal_entity = response.parse()
        assert_matches_type(SyncPage[LegalEntity], legal_entity, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.legal_entities.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            legal_entity = response.parse()
            assert_matches_type(SyncPage[LegalEntity], legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncLegalEntities:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncModernTreasury) -> None:
        legal_entity = await async_client.legal_entities.create(
            legal_entity_type="business",
        )
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        legal_entity = await async_client.legal_entities.create(
            legal_entity_type="business",
            addresses=[
                {
                    "country": "country",
                    "line1": "line1",
                    "locality": "locality",
                    "postal_code": "postal_code",
                    "region": "region",
                    "address_types": ["business"],
                    "line2": "line2",
                }
            ],
            bank_settings={
                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "backup_withholding_percentage": 0,
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "enable_backup_withholding": True,
                "live_mode": True,
                "object": "object",
                "privacy_opt_out": True,
                "regulation_o": True,
                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            business_name="business_name",
            citizenship_country="citizenship_country",
            date_formed=parse_date("2019-12-27"),
            date_of_birth=parse_date("2019-12-27"),
            doing_business_as_names=["string"],
            email="email",
            first_name="first_name",
            identifications=[
                {
                    "id_number": "id_number",
                    "id_type": "ar_cuil",
                    "issuing_country": "issuing_country",
                }
            ],
            last_name="last_name",
            legal_entity_associations=[
                {
                    "relationship_types": ["beneficial_owner"],
                    "child_legal_entity": {
                        "addresses": [
                            {
                                "country": "country",
                                "line1": "line1",
                                "locality": "locality",
                                "postal_code": "postal_code",
                                "region": "region",
                                "address_types": ["business"],
                                "line2": "line2",
                            }
                        ],
                        "bank_settings": {
                            "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "backup_withholding_percentage": 0,
                            "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "enable_backup_withholding": True,
                            "live_mode": True,
                            "object": "object",
                            "privacy_opt_out": True,
                            "regulation_o": True,
                            "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                        },
                        "business_name": "business_name",
                        "citizenship_country": "citizenship_country",
                        "date_formed": parse_date("2019-12-27"),
                        "date_of_birth": parse_date("2019-12-27"),
                        "doing_business_as_names": ["string"],
                        "email": "email",
                        "first_name": "first_name",
                        "identifications": [
                            {
                                "id_number": "id_number",
                                "id_type": "ar_cuil",
                                "issuing_country": "issuing_country",
                            }
                        ],
                        "last_name": "last_name",
                        "legal_entity_type": "business",
                        "legal_structure": "corporation",
                        "metadata": {
                            "key": "value",
                            "foo": "bar",
                            "modern": "treasury",
                        },
                        "middle_name": "middle_name",
                        "phone_numbers": [{"phone_number": "phone_number"}],
                        "politically_exposed_person": True,
                        "preferred_name": "preferred_name",
                        "prefix": "prefix",
                        "risk_rating": "low",
                        "suffix": "suffix",
                        "wealth_and_employment_details": {
                            "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "annual_income": 0,
                            "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "employer_country": "employer_country",
                            "employer_name": "employer_name",
                            "employer_state": "employer_state",
                            "employment_status": "employed",
                            "income_country": "income_country",
                            "income_source": "family_support",
                            "income_state": "income_state",
                            "industry": "accounting",
                            "live_mode": True,
                            "object": "object",
                            "occupation": "consulting",
                            "source_of_funds": "alimony",
                            "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "wealth_source": "business_sale",
                        },
                        "website": "website",
                    },
                    "child_legal_entity_id": "child_legal_entity_id",
                    "ownership_percentage": 0,
                    "title": "title",
                }
            ],
            legal_structure="corporation",
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
            middle_name="middle_name",
            phone_numbers=[{"phone_number": "phone_number"}],
            politically_exposed_person=True,
            preferred_name="preferred_name",
            prefix="prefix",
            risk_rating="low",
            suffix="suffix",
            wealth_and_employment_details={
                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "annual_income": 0,
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "employer_country": "employer_country",
                "employer_name": "employer_name",
                "employer_state": "employer_state",
                "employment_status": "employed",
                "income_country": "income_country",
                "income_source": "family_support",
                "income_state": "income_state",
                "industry": "accounting",
                "live_mode": True,
                "object": "object",
                "occupation": "consulting",
                "source_of_funds": "alimony",
                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "wealth_source": "business_sale",
            },
            website="website",
        )
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.legal_entities.with_raw_response.create(
            legal_entity_type="business",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        legal_entity = response.parse()
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.legal_entities.with_streaming_response.create(
            legal_entity_type="business",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            legal_entity = await response.parse()
            assert_matches_type(LegalEntity, legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        legal_entity = await async_client.legal_entities.retrieve(
            "id",
        )
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.legal_entities.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        legal_entity = response.parse()
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.legal_entities.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            legal_entity = await response.parse()
            assert_matches_type(LegalEntity, legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.legal_entities.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncModernTreasury) -> None:
        legal_entity = await async_client.legal_entities.update(
            id="id",
        )
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        legal_entity = await async_client.legal_entities.update(
            id="id",
            addresses=[
                {
                    "country": "country",
                    "line1": "line1",
                    "locality": "locality",
                    "postal_code": "postal_code",
                    "region": "region",
                    "address_types": ["business"],
                    "line2": "line2",
                }
            ],
            bank_settings={
                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "backup_withholding_percentage": 0,
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "enable_backup_withholding": True,
                "live_mode": True,
                "object": "object",
                "privacy_opt_out": True,
                "regulation_o": True,
                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            business_name="business_name",
            citizenship_country="citizenship_country",
            date_formed=parse_date("2019-12-27"),
            date_of_birth=parse_date("2019-12-27"),
            doing_business_as_names=["string"],
            email="email",
            first_name="first_name",
            identifications=[
                {
                    "id_number": "id_number",
                    "id_type": "ar_cuil",
                    "issuing_country": "issuing_country",
                }
            ],
            last_name="last_name",
            legal_structure="corporation",
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
            middle_name="middle_name",
            phone_numbers=[{"phone_number": "phone_number"}],
            politically_exposed_person=True,
            preferred_name="preferred_name",
            prefix="prefix",
            risk_rating="low",
            suffix="suffix",
            wealth_and_employment_details={
                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "annual_income": 0,
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "employer_country": "employer_country",
                "employer_name": "employer_name",
                "employer_state": "employer_state",
                "employment_status": "employed",
                "income_country": "income_country",
                "income_source": "family_support",
                "income_state": "income_state",
                "industry": "accounting",
                "live_mode": True,
                "object": "object",
                "occupation": "consulting",
                "source_of_funds": "alimony",
                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "wealth_source": "business_sale",
            },
            website="website",
        )
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.legal_entities.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        legal_entity = response.parse()
        assert_matches_type(LegalEntity, legal_entity, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.legal_entities.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            legal_entity = await response.parse()
            assert_matches_type(LegalEntity, legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.legal_entities.with_raw_response.update(
                id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        legal_entity = await async_client.legal_entities.list()
        assert_matches_type(AsyncPage[LegalEntity], legal_entity, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        legal_entity = await async_client.legal_entities.list(
            after_cursor="after_cursor",
            legal_entity_type="business",
            metadata={"foo": "string"},
            per_page=0,
            show_deleted="show_deleted",
        )
        assert_matches_type(AsyncPage[LegalEntity], legal_entity, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.legal_entities.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        legal_entity = response.parse()
        assert_matches_type(AsyncPage[LegalEntity], legal_entity, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.legal_entities.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            legal_entity = await response.parse()
            assert_matches_type(AsyncPage[LegalEntity], legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.pagination import SyncPage, AsyncPage
from modern_treasury.types.invoices import (
    InvoiceLineItem,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestLineItems:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: ModernTreasury) -> None:
        line_item = client.invoices.line_items.create(
            invoice_id="invoice_id",
            name="name",
            unit_amount=0,
        )
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: ModernTreasury) -> None:
        line_item = client.invoices.line_items.create(
            invoice_id="invoice_id",
            name="name",
            unit_amount=0,
            description="description",
            direction="direction",
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
            quantity=0,
            unit_amount_decimal="unit_amount_decimal",
        )
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: ModernTreasury) -> None:
        response = client.invoices.line_items.with_raw_response.create(
            invoice_id="invoice_id",
            name="name",
            unit_amount=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        line_item = response.parse()
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: ModernTreasury) -> None:
        with client.invoices.line_items.with_streaming_response.create(
            invoice_id="invoice_id",
            name="name",
            unit_amount=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            line_item = response.parse()
            assert_matches_type(InvoiceLineItem, line_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `invoice_id` but received ''"):
            client.invoices.line_items.with_raw_response.create(
                invoice_id="",
                name="name",
                unit_amount=0,
            )

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        line_item = client.invoices.line_items.retrieve(
            id="id",
            invoice_id="invoice_id",
        )
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.invoices.line_items.with_raw_response.retrieve(
            id="id",
            invoice_id="invoice_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        line_item = response.parse()
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.invoices.line_items.with_streaming_response.retrieve(
            id="id",
            invoice_id="invoice_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            line_item = response.parse()
            assert_matches_type(InvoiceLineItem, line_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `invoice_id` but received ''"):
            client.invoices.line_items.with_raw_response.retrieve(
                id="id",
                invoice_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.invoices.line_items.with_raw_response.retrieve(
                id="",
                invoice_id="invoice_id",
            )

    @parametrize
    def test_method_update(self, client: ModernTreasury) -> None:
        line_item = client.invoices.line_items.update(
            id="id",
            invoice_id="invoice_id",
        )
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: ModernTreasury) -> None:
        line_item = client.invoices.line_items.update(
            id="id",
            invoice_id="invoice_id",
            description="description",
            direction="direction",
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
            name="name",
            quantity=0,
            unit_amount=0,
            unit_amount_decimal="unit_amount_decimal",
        )
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: ModernTreasury) -> None:
        response = client.invoices.line_items.with_raw_response.update(
            id="id",
            invoice_id="invoice_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        line_item = response.parse()
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: ModernTreasury) -> None:
        with client.invoices.line_items.with_streaming_response.update(
            id="id",
            invoice_id="invoice_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            line_item = response.parse()
            assert_matches_type(InvoiceLineItem, line_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `invoice_id` but received ''"):
            client.invoices.line_items.with_raw_response.update(
                id="id",
                invoice_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.invoices.line_items.with_raw_response.update(
                id="",
                invoice_id="invoice_id",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        line_item = client.invoices.line_items.list(
            invoice_id="invoice_id",
        )
        assert_matches_type(SyncPage[InvoiceLineItem], line_item, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        line_item = client.invoices.line_items.list(
            invoice_id="invoice_id",
            after_cursor="after_cursor",
            per_page=0,
        )
        assert_matches_type(SyncPage[InvoiceLineItem], line_item, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.invoices.line_items.with_raw_response.list(
            invoice_id="invoice_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        line_item = response.parse()
        assert_matches_type(SyncPage[InvoiceLineItem], line_item, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.invoices.line_items.with_streaming_response.list(
            invoice_id="invoice_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            line_item = response.parse()
            assert_matches_type(SyncPage[InvoiceLineItem], line_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `invoice_id` but received ''"):
            client.invoices.line_items.with_raw_response.list(
                invoice_id="",
            )

    @parametrize
    def test_method_delete(self, client: ModernTreasury) -> None:
        line_item = client.invoices.line_items.delete(
            id="id",
            invoice_id="invoice_id",
        )
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: ModernTreasury) -> None:
        response = client.invoices.line_items.with_raw_response.delete(
            id="id",
            invoice_id="invoice_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        line_item = response.parse()
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: ModernTreasury) -> None:
        with client.invoices.line_items.with_streaming_response.delete(
            id="id",
            invoice_id="invoice_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            line_item = response.parse()
            assert_matches_type(InvoiceLineItem, line_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `invoice_id` but received ''"):
            client.invoices.line_items.with_raw_response.delete(
                id="id",
                invoice_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.invoices.line_items.with_raw_response.delete(
                id="",
                invoice_id="invoice_id",
            )


class TestAsyncLineItems:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncModernTreasury) -> None:
        line_item = await async_client.invoices.line_items.create(
            invoice_id="invoice_id",
            name="name",
            unit_amount=0,
        )
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        line_item = await async_client.invoices.line_items.create(
            invoice_id="invoice_id",
            name="name",
            unit_amount=0,
            description="description",
            direction="direction",
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
            quantity=0,
            unit_amount_decimal="unit_amount_decimal",
        )
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.invoices.line_items.with_raw_response.create(
            invoice_id="invoice_id",
            name="name",
            unit_amount=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        line_item = response.parse()
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.invoices.line_items.with_streaming_response.create(
            invoice_id="invoice_id",
            name="name",
            unit_amount=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            line_item = await response.parse()
            assert_matches_type(InvoiceLineItem, line_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `invoice_id` but received ''"):
            await async_client.invoices.line_items.with_raw_response.create(
                invoice_id="",
                name="name",
                unit_amount=0,
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        line_item = await async_client.invoices.line_items.retrieve(
            id="id",
            invoice_id="invoice_id",
        )
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.invoices.line_items.with_raw_response.retrieve(
            id="id",
            invoice_id="invoice_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        line_item = response.parse()
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.invoices.line_items.with_streaming_response.retrieve(
            id="id",
            invoice_id="invoice_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            line_item = await response.parse()
            assert_matches_type(InvoiceLineItem, line_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `invoice_id` but received ''"):
            await async_client.invoices.line_items.with_raw_response.retrieve(
                id="id",
                invoice_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.invoices.line_items.with_raw_response.retrieve(
                id="",
                invoice_id="invoice_id",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncModernTreasury) -> None:
        line_item = await async_client.invoices.line_items.update(
            id="id",
            invoice_id="invoice_id",
        )
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        line_item = await async_client.invoices.line_items.update(
            id="id",
            invoice_id="invoice_id",
            description="description",
            direction="direction",
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
            name="name",
            quantity=0,
            unit_amount=0,
            unit_amount_decimal="unit_amount_decimal",
        )
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.invoices.line_items.with_raw_response.update(
            id="id",
            invoice_id="invoice_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        line_item = response.parse()
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.invoices.line_items.with_streaming_response.update(
            id="id",
            invoice_id="invoice_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            line_item = await response.parse()
            assert_matches_type(InvoiceLineItem, line_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `invoice_id` but received ''"):
            await async_client.invoices.line_items.with_raw_response.update(
                id="id",
                invoice_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.invoices.line_items.with_raw_response.update(
                id="",
                invoice_id="invoice_id",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        line_item = await async_client.invoices.line_items.list(
            invoice_id="invoice_id",
        )
        assert_matches_type(AsyncPage[InvoiceLineItem], line_item, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        line_item = await async_client.invoices.line_items.list(
            invoice_id="invoice_id",
            after_cursor="after_cursor",
            per_page=0,
        )
        assert_matches_type(AsyncPage[InvoiceLineItem], line_item, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.invoices.line_items.with_raw_response.list(
            invoice_id="invoice_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        line_item = response.parse()
        assert_matches_type(AsyncPage[InvoiceLineItem], line_item, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.invoices.line_items.with_streaming_response.list(
            invoice_id="invoice_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            line_item = await response.parse()
            assert_matches_type(AsyncPage[InvoiceLineItem], line_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `invoice_id` but received ''"):
            await async_client.invoices.line_items.with_raw_response.list(
                invoice_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncModernTreasury) -> None:
        line_item = await async_client.invoices.line_items.delete(
            id="id",
            invoice_id="invoice_id",
        )
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.invoices.line_items.with_raw_response.delete(
            id="id",
            invoice_id="invoice_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        line_item = response.parse()
        assert_matches_type(InvoiceLineItem, line_item, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.invoices.line_items.with_streaming_response.delete(
            id="id",
            invoice_id="invoice_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            line_item = await response.parse()
            assert_matches_type(InvoiceLineItem, line_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `invoice_id` but received ''"):
            await async_client.invoices.line_items.with_raw_response.delete(
                id="id",
                invoice_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.invoices.line_items.with_raw_response.delete(
                id="",
                invoice_id="invoice_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CounterpartyListParams"]


class CounterpartyListParams(TypedDict, total=False):
    after_cursor: Optional[str]

    created_at_lower_bound: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Used to return counterparties created after some datetime."""

    created_at_upper_bound: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Used to return counterparties created before some datetime."""

    email: str
    """Performs a partial string match of the email field.

    This is also case insensitive.
    """

    legal_entity_id: str
    """Filters for counterparties with the given legal entity ID."""

    metadata: Dict[str, str]
    """
    For example, if you want to query for records with metadata key `Type` and value
    `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
    parameters.
    """

    name: str
    """Performs a partial string match of the name field.

    This is also case insensitive.
    """

    per_page: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional

import httpx

from .. import _legacy_response
from ..types import ledgerable_event_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from .._base_client import make_request_options
from ..types.ledgerable_event import LedgerableEvent

__all__ = ["LedgerableEvents", "AsyncLedgerableEvents"]


class LedgerableEvents(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LedgerableEventsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return LedgerableEventsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LedgerableEventsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return LedgerableEventsWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        custom_data: Optional[object] | NotGiven = NOT_GIVEN,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerableEvent:
        """
        Create a ledgerable event.

        Args:
          name: Name of the ledgerable event.

          custom_data: Additionally data to be used by the Ledger Event Handler.

          description: Description of the ledgerable event.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/api/ledgerable_events",
            body=maybe_transform(
                {
                    "name": name,
                    "custom_data": custom_data,
                    "description": description,
                    "metadata": metadata,
                },
                ledgerable_event_create_params.LedgerableEventCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerableEvent,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LedgerableEvent:
        """
        Get details on a single ledgerable event.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/ledgerable_events/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LedgerableEvent,
        )


class AsyncLedgerableEvents(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLedgerableEventsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLedgerableEventsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLedgerableEventsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncLedgerableEventsWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        custom_data: Optional[object] | NotGiven = NOT_GIVEN,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerableEvent:
        """
        Create a ledgerable event.

        Args:
          name: Name of the ledgerable event.

          custom_data: Additionally data to be used by the Ledger Event Handler.

          description: Description of the ledgerable event.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/api/ledgerable_events",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "custom_data": custom_data,
                    "description": description,
                    "metadata": metadata,
                },
                ledgerable_event_create_params.LedgerableEventCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerableEvent,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LedgerableEvent:
        """
        Get details on a single ledgerable event.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/ledgerable_events/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LedgerableEvent,
        )


class LedgerableEventsWithRawResponse:
    def __init__(self, ledgerable_events: LedgerableEvents) -> None:
        self._ledgerable_events = ledgerable_events

        self.create = _legacy_response.to_raw_response_wrapper(
            ledgerable_events.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            ledgerable_events.retrieve,
        )


class AsyncLedgerableEventsWithRawResponse:
    def __init__(self, ledgerable_events: AsyncLedgerableEvents) -> None:
        self._ledgerable_events = ledgerable_events

        self.create = _legacy_response.async_to_raw_response_wrapper(
            ledgerable_events.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            ledgerable_events.retrieve,
        )


class LedgerableEventsWithStreamingResponse:
    def __init__(self, ledgerable_events: LedgerableEvents) -> None:
        self._ledgerable_events = ledgerable_events

        self.create = to_streamed_response_wrapper(
            ledgerable_events.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            ledgerable_events.retrieve,
        )


class AsyncLedgerableEventsWithStreamingResponse:
    def __init__(self, ledgerable_events: AsyncLedgerableEvents) -> None:
        self._ledgerable_events = ledgerable_events

        self.create = async_to_streamed_response_wrapper(
            ledgerable_events.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            ledgerable_events.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypedDict

__all__ = ["LedgerTransactionListParams", "OrderBy"]


class LedgerTransactionListParams(TypedDict, total=False):
    id: List[str]
    """
    If you have specific IDs to retrieve in bulk, you can pass them as query
    parameters delimited with `id[]=`, for example `?id[]=123&id[]=abc`.
    """

    after_cursor: Optional[str]

    effective_at: Dict[str, Union[str, datetime]]
    """
    Use "gt" (>), "gte" (>=), "lt" (<), "lte" (<=), or "eq" (=) to filter by
    effective at. For example, for all transactions after Jan 1 2000, use
    effective_at%5Bgt%5D=2000-01-01T00:00:00:00.000Z.
    """

    effective_date: Dict[str, Union[str, datetime]]
    """
    Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by
    effective date. For example, for all dates after Jan 1 2000, use
    effective_date%5Bgt%5D=2000-01-01.
    """

    external_id: str

    ledger_account_category_id: str

    ledger_account_id: str

    ledger_account_settlement_id: str

    ledger_id: str

    ledgerable_id: str

    ledgerable_type: Literal[
        "expected_payment", "incoming_payment_detail", "paper_item", "payment_order", "return", "reversal"
    ]

    metadata: Dict[str, str]
    """
    For example, if you want to query for records with metadata key `Type` and value
    `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
    parameters.
    """

    order_by: OrderBy
    """Order by `created_at` or `effective_at` in `asc` or `desc` order.

    For example, to order by `effective_at asc`, use
    `order_by%5Beffective_at%5D=asc`. Ordering by only one field at a time is
    supported.
    """

    per_page: int

    posted_at: Dict[str, Union[str, datetime]]
    """
    Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by the
    posted at timestamp. For example, for all times after Jan 1 2000 12:00 UTC, use
    posted_at%5Bgt%5D=2000-01-01T12:00:00Z.
    """

    reverses_ledger_transaction_id: str

    status: Literal["pending", "posted", "archived"]

    updated_at: Dict[str, Union[str, datetime]]
    """
    Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by the
    posted at timestamp. For example, for all times after Jan 1 2000 12:00 UTC, use
    updated_at%5Bgt%5D=2000-01-01T12:00:00Z.
    """


class OrderBy(TypedDict, total=False):
    created_at: Literal["asc", "desc"]

    effective_at: Literal["asc", "desc"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from .._compat import PYDANTIC_V2
from .._models import BaseModel
from .transaction import Transaction
from .expected_payment import ExpectedPayment
from .ledger_transaction import LedgerTransaction

__all__ = ["BulkResult", "Entity", "EntityBulkError", "EntityBulkErrorRequestErrors", "EntityBulkErrorRequestError"]


class EntityBulkErrorRequestError(BaseModel):
    code: Optional[str] = None

    message: Optional[str] = None

    parameter: Optional[str] = None


EntityBulkErrorRequestErrors = EntityBulkErrorRequestError
"""This type is deprecated and will be removed in a future release.

Please use EntityBulkErrorRequestError instead.
"""


class EntityBulkError(BaseModel):
    id: str

    created_at: datetime

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    request_errors: List[EntityBulkErrorRequestError]

    updated_at: datetime


Entity: TypeAlias = Union["PaymentOrder", ExpectedPayment, LedgerTransaction, Transaction, EntityBulkError]


class BulkResult(BaseModel):
    id: str

    created_at: datetime

    entity: Entity
    """An object with type as indicated by `entity_type`.

    This is the result object that is generated by performing the requested action
    on the provided input `request_params`.
    """

    entity_id: str
    """Unique identifier for the result entity object."""

    entity_type: Literal["payment_order", "ledger_transaction", "transaction", "expected_payment", "bulk_error"]
    """The type of the result entity object.

    For a successful bulk result, this is the same as the `resource_type` of the
    bulk request. For a failed bulk result, this is always bulk_error
    """

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    request_id: str
    """Unique identifier for the request that created this bulk result.

    This is the ID of the bulk request when `request_type` is bulk_request
    """

    request_params: Optional[Dict[str, str]] = None
    """
    An optional object that contains the provided input params for the request that
    created this result. This is an item in the `resources` array for the
    bulk_request
    """

    request_type: Literal["bulk_request"]
    """The type of the request that created this result.

    bulk_request is the only supported `request_type`
    """

    status: Literal["pending", "successful", "failed"]
    """One of successful or failed."""

    updated_at: datetime


from .payment_order import PaymentOrder

if PYDANTIC_V2:
    BulkResult.model_rebuild()
    EntityBulkError.model_rebuild()
    EntityBulkErrorRequestError.model_rebuild()
else:
    BulkResult.update_forward_refs()  # type: ignore
    EntityBulkError.update_forward_refs()  # type: ignore
    EntityBulkErrorRequestError.update_forward_refs()  # type: ignore
